@@ -3,7 +3,7 @@ package Git::Code::Review::Utilities;
 use strict;
 use warnings;
 
-our $VERSION = '2.5'; # VERSION
+our $VERSION = '2.6'; # VERSION
 
 # Utility Modules
 use CLI::Helpers qw(:all);
@@ -33,9 +33,11 @@ use Sub::Exporter -setup => {
         gcr_profile
         gcr_profiles
         gcr_valid_profile
+        gcr_load_profile
         gcr_open_editor
         gcr_view_commit
         gcr_view_commit_files
+        gcr_view_readme
         gcr_change_profile
         gcr_change_state
         gcr_not_resigned
@@ -142,6 +144,36 @@ sub gcr_valid_profile {
     return exists $_profiles{$profile};
 }
 
+
+sub gcr_load_profile {
+    my ($profile) = @_;
+    # Select everything if there's no profiles
+    my %profile = ();
+
+    # Selection Config for the Profile
+    my $select_file = File::Spec->catfile( $AUDITDIR, qw( .code-review profiles ), $profile, 'selection.yaml' );
+    if( -f $select_file ) {
+        my $data;
+        eval {
+            $data = YAML::LoadFile($select_file);
+        };
+        if( my $err = $@ ) {
+            output({stderr=>1,color=>'red'}, "Error loading profile YAML: $err");
+            exit 1;
+        }
+        else {
+           %profile = %{ $data };
+        }
+    }
+    elsif($profile eq 'default') {
+        %profile = ( path => [qw(**)], );
+    }
+    die "error loading selection criteria for $profile" unless scalar(keys %profile);
+
+    return wantarray ? %profile : \%profile;
+}
+
+
 my %_config = ();
 sub gcr_config {
 
@@ -160,7 +192,7 @@ sub gcr_config {
             $_config{$gitrc{$k}} = $v;
         }
 
-        foreach my $sub (qw(notification mailhandler)) {
+        foreach my $sub (qw(notification mailhandler review)) {
             my @files = (
                 File::Spec->catfile($AUDITDIR,'.code-review',"${sub}.config"),
                 File::Spec->catfile($AUDITDIR,qw(.code-review profiles),gcr_profile(exists => 0),"${sub}.config")
@@ -436,6 +468,28 @@ sub gcr_view_commit_files {
 }
 
 
+sub gcr_global_readme {
+    return File::Spec->catfile( $AUDITDIR, '.code-review', 'README');
+}
+
+
+sub gcr_profile_readme {
+    return File::Spec->catfile( $AUDITDIR, qw( .code-review profiles ), gcr_profile( exists => 0 ), 'README' );
+}
+
+
+sub gcr_readme {
+    my $readme = gcr_profile_readme();
+    return $readme if -e $readme;
+    return gcr_global_readme(); # it is possible that the global README file may not exist
+}
+
+
+sub gcr_view_readme {
+    gcr_open_editor( readonly => gcr_readme() );
+}
+
+
 
 sub gcr_change_profile {
     my($commit,$profile,$info) = @_;
@@ -754,22 +808,36 @@ sub _get_commit_date {
 }
 
 my %_selections = ();
+my $_selections_cached = 0;
 sub _get_commit_select_date {
     my ($sha1) = @_;
+    return $_selections{ $sha1 } if exists $_selections{ $sha1 };
 
     # Better cache this once per run
-    if(!keys %_selections) {
-        my $audit = gcr_repo();
-        my @log_options = qw(--reverse -F --grep);
-        push @log_options, "state: select";
+    my $audit = gcr_repo();
+    if( ! $_selections_cached ) {
+        if(scalar keys %_selections <= 1 ) {
+            # does not cache all commits, just tries to resolve a single one
+            my @log_options = ( qw( --reverse -n 1 -F --grep ), "state: select", '--', sprintf "*/%s**", $sha1 );
+            my $logs = $audit->log( @log_options );
+            if ( my $log = $logs->next ) {
+                my @commits = map { _get_sha1( basename( $_ ) ) } grep { /\.patch$/ } $audit->run( qw( diff-tree --no-commit-id --name-only -r ), $log->commit );
+                return $_selections{$sha1} = strftime( '%F', localtime( $log->author_localtime ) ) if grep { $_ eq $sha1 } @commits;
+            }
+            return $_selections{$sha1} = '1970-01-01';
+        } else {
+            $_selections_cached++;
+            my @log_options = qw(--reverse -F --grep);
+            push @log_options, "state: select";
 
-        my $logs = $audit->log(@log_options);
-        while(my $log = $logs->next) {
-            my $date = strftime( '%F', localtime($log->author_localtime));
-            my @commits = map { _get_sha1(basename($_)) } grep { /\.patch$/ } $audit->run(qw(diff-tree --no-commit-id --name-only -r), $log->commit);
-            foreach my $commit (@commits) {
-                next if exists $_selections{$commit};
-                $_selections{$commit} = $date;
+            my $logs = $audit->log(@log_options);
+            while(my $log = $logs->next) {
+                my $date = strftime( '%F', localtime($log->author_localtime));
+                my @commits = map { _get_sha1(basename($_)) } grep { /\.patch$/ } $audit->run(qw(diff-tree --no-commit-id --name-only -r), $log->commit);
+                foreach my $commit (@commits) {
+                    next if exists $_selections{$commit};
+                    $_selections{$commit} = $date;
+                }
             }
         }
     }
@@ -839,7 +907,7 @@ Git::Code::Review::Utilities - Tools for performing code review using Git as the
 
 =head1 VERSION
 
-version 2.5
+version 2.6
 
 =head1 FUNCTIONS
 
@@ -864,6 +932,10 @@ Returns a list of profiles with a selection file available
 
 Returns 1 if a valid profile and undef if not
 
+=head2 gcr_load_profile($profile_name)
+
+Returns selection config for the profile
+
 =head2 gcr_config()
 
 Returns a copy of our configuration as a hash or hash ref.
@@ -919,6 +991,22 @@ stores time spent in editor as review_time in the hash.
 Display a menu containing the files mentioned in the commit with the ability to view
 the contents of one of those files.
 
+=head2 gcr_global_readme()
+
+Get the global readme file.
+
+=head2 gcr_profile_readme()
+
+Get the readme file for a profile.
+
+=head2 gcr_readme()
+
+Get the readme file for the current profile if it exists or a global one.
+
+=head2 gcr_view_readme()
+
+View the README file for the current profile if it exists or the global README file.
+
 =head2 gcr_change_profile($commit_info,$profile,$details)
 
 $commit_info is a hash attained from gcr_commit_info()
@@ -987,9 +1075,9 @@ Figure out the review path from a file path.
 
 Figure out the commit date.
 
-=head2 _get_commit_date($path)
+=head2 _get_commit_select_date($path)
 
-Figure out the commit date.
+Figure out the commit selection date.
 
 =head2 _get_commit_profile($path)
 
